/*
 * Project thermo_15_vsc
 * Description: I2C communication example with Thermo 15 click
 * Author: Luka Filipovic
 * Date: 29.11.2019.
 */

#include "stdint.h"
#include <Wire.h>

#define _MIKROBUS_1
// #define _MIKROBUS_2

#ifdef _MIKROBUS_1
const uint8_t AN_PIN = 19;
const uint8_t RST_PIN = 17;
const uint8_t CS_PIN = 14;
const uint8_t SCK_PIN = 13;
const uint8_t MISO_PIN = 11;
const uint8_t MOSI_PIN = 12;
const uint8_t PWM_PIN = 16;
const uint8_t INT_PIN = 15;
const uint8_t RX_PIN = 10;
const uint8_t TX_PIN = 9;
const uint8_t SCL_PIN = 1;
const uint8_t SDA_PIN = 0;
#endif

#ifdef _MIKROBUS_2
const uint8_t AN_PIN = 18;
const uint8_t RST_PIN = 7;
const uint8_t CS_PIN = 5;
const uint8_t SCK_PIN = 13;
const uint8_t MISO_PIN = 11;
const uint8_t MOSI_PIN = 12;
const uint8_t PWM_PIN = 8;
const uint8_t INT_PIN = 6;
const uint8_t RX_PIN = 10;
const uint8_t TX_PIN = 9;
const uint8_t SCL_PIN = 1;
const uint8_t SDA_PIN = 0;
#endif


/* Device Slave address */
const uint8_t _THERMO15_DEVICE_SLAVE_ADDR_000 = 0x48;
const uint8_t _THERMO15_DEVICE_SLAVE_ADDR_001 = 0x49;
const uint8_t _THERMO15_DEVICE_SLAVE_ADDR_010 = 0x4A;
const uint8_t _THERMO15_DEVICE_SLAVE_ADDR_011 = 0x4B;
const uint8_t _THERMO15_DEVICE_SLAVE_ADDR_100 = 0x4C;
const uint8_t _THERMO15_DEVICE_SLAVE_ADDR_101 = 0x4D;
const uint8_t _THERMO15_DEVICE_SLAVE_ADDR_110 = 0x4E;
const uint8_t _THERMO15_DEVICE_SLAVE_ADDR_111 = 0x4F;

/* Registar */
const uint8_t _THERMO15_REG_STORED_TEMPERATURE     = 0x00;
const uint8_t _THERMO15_REG_CONFIGURATION          = 0x01;
const uint8_t _THERMO15_REG_TEMPERATURE_HYSTERESIS = 0x02;
const uint8_t _THERMO15_REG_TEMPERATURE_LIMIT      = 0x03;
const uint8_t _THERMO15_REG_ONE_SHOT               = 0x04;

/* Default value in the register */
const uint8_t _THERMO15_DEF_TEMP_HYSTERESIS = 75;
const uint8_t _THERMO15_DEF_TEMP_LIMIT      = 80;

/* Configurations */
const uint8_t _THERMO15_CFG_WMODE_NORMAL      = 0x00;
const uint8_t _THERMO15_CFG_WMODE_ONE_SHOT    = 0x20;
const uint8_t _THERMO15_CFG_WMODE_SHUTDOWN    = 0x01;
const uint8_t _THERMO15_CFG_FAULT_QUEUE_1     = 0x00;
const uint8_t _THERMO15_CFG_FAULT_QUEUE_2     = 0x08;
const uint8_t _THERMO15_CFG_FAULT_QUEUE_4     = 0x10;
const uint8_t _THERMO15_CFG_FAULT_QUEUE_6     = 0x18;
const uint8_t _THERMO15_CFG_ALERT_ACTIVE_LOW  = 0x00;
const uint8_t _THERMO15_CFG_ALERT_ACTIVE_HIGH = 0x04;
const uint8_t _THERMO15_CFG_TMODE_COMPARATOR  = 0x00;
const uint8_t _THERMO15_CFG_TMODE_INTERRUPT   = 0x02;

/* Temperature data in .. */
const uint8_t _THERMO15_TEMP_IN_CELSIUS    = 0x00;
const uint8_t _THERMO15_TEMP_IN_KELVIN     = 0x01;
const uint8_t _THERMO15_TEMP_IN_FAHRENHEIT = 0x02;


static uint8_t _slaveAddress = _THERMO15_DEVICE_SLAVE_ADDR_000;

static int hal_i2cWrite(uint8_t slaveAddress, uint8_t *pBuf, uint16_t nBytes, uint8_t endMode)
{
    int res = 0;
    uint8_t *ptr = pBuf;
    
    Wire.beginTransmission(byte(slaveAddress));

    while(nBytes--)
        Wire.write(byte(*(ptr++)));

    Wire.endTransmission();
    return res;
}

static int hal_i2cRead(uint8_t slaveAddress, uint8_t *pBuf, uint16_t nBytes, uint8_t endMode)
{
    int res = 0;
    uint8_t *ptr = pBuf;

    Wire.requestFrom(byte(slaveAddress), nBytes);

    while(nBytes--)
        *(ptr++) = Wire.read();

    return res;
}

uint8_t thermo15_readByte(uint8_t regAddr)
{
    uint8_t writeData[ 1 ];
    uint8_t readData[ 1 ];
    
    writeData[ 0 ] = regAddr;

    hal_i2cWrite(_slaveAddress, writeData, 1, 1);
    hal_i2cRead(_slaveAddress, readData, 1, 1);
    return readData[ 0 ];
}

uint16_t thermo15_readData(uint8_t regAddr)
{
    uint8_t writeData[ 1 ];
    uint8_t readData[ 2 ];
    uint16_t rawData;
    int i = 2;
    
    writeData[ 0 ] = regAddr;

    hal_i2cWrite(_slaveAddress, writeData, 1, 1);
    hal_i2cRead(_slaveAddress, readData, 2, 1);

    rawData = readData[ 0 ];
    rawData <<= 8;
    rawData |= readData[ 1 ];
    
    return rawData;
}

void thermo15_configuration(uint8_t cfgData)
{
    uint8_t writeData[ 2 ];

    writeData[ 0 ] = _THERMO15_REG_CONFIGURATION;
    writeData[ 1 ] = cfgData;

  hal_i2cWrite(_slaveAddress, writeData, 2, 1);


}

float thermo15_getAmbientTemperatureData(uint8_t tempIn)
{
    uint16_t rawTemp;
    float Temperature;
    
    rawTemp =  thermo15_readData(_THERMO15_REG_STORED_TEMPERATURE);
    rawTemp >>= 4;
    
    if(rawTemp > 0x1000)
    {
        Temperature = (float)((rawTemp - 4096) / 16.0);
    }
    else
    {
        Temperature = (float)(rawTemp / 16.0);
    }
    
    delay(100);
    
    if(tempIn == _THERMO15_TEMP_IN_KELVIN)
    {
        Temperature = Temperature + 273.15;
    }
    else if(tempIn == _THERMO15_TEMP_IN_FAHRENHEIT)
    {
        Temperature = (Temperature * 9.0/5.0) + 32;
    }
    
    return Temperature;
}

void thermo15_setTempRegister(uint8_t tempReg, float temp)
{
    uint16_t TempData;
    uint8_t writeData[ 3 ];

    TempData = (uint16_t)(temp * 16);
    TempData <<= 4;
    
    writeData[ 0 ] = tempReg;
    writeData[ 1 ] = (TempData >> 8);
    writeData[ 2 ] = (TempData & 0x00FF);
    
    hal_i2cWrite(_slaveAddress, writeData, 3, 1);
  
}

float thermo15_getTempRegister(uint8_t tempReg)
{
    float Temperature;
    uint16_t TempData;

    TempData = thermo15_readData( tempReg );
    TempData >>= 4;
    
    Temperature = (float)(TempData / 16.0);
    
    return Temperature;
}

char demoText[50] = {0};
    
void systemInit()
{
    Wire.begin();
    Serial.begin(9600);
    Serial.println( " --- System Init --- ");
    delay(100);
}

void applicationInit()
{
    float TempHyst;
    float TempLimit;

    /* Configuration */
    thermo15_configuration( _THERMO15_CFG_WMODE_NORMAL | 
                            _THERMO15_CFG_FAULT_QUEUE_1 |
                            _THERMO15_CFG_ALERT_ACTIVE_HIGH | 
                            _THERMO15_CFG_TMODE_COMPARATOR );
                            
    /* Temperature Register */
    Serial.println( " --- Temperature register data --- ");
    Serial.println( "  ");
    
    TempHyst = thermo15_getTempRegister( _THERMO15_REG_TEMPERATURE_HYSTERESIS );
    String(TempHyst).toCharArray(demoText,10);
    Serial.print("** Temp - Hysteresis  : ");
    Serial.print(demoText);
    Serial.println(" C ");

    TempLimit = thermo15_getTempRegister( _THERMO15_REG_TEMPERATURE_LIMIT );
    String(TempLimit).toCharArray(demoText,10);
    Serial.println("** Temp - Limit  : ");
    Serial.println(demoText);
    Serial.println(" C ");

    Serial.println("  ");
    Serial.println( " --- Ambient temperature measurement --- ");
}

void applicationTask()
{
    float Temperature;
    
    Temperature = thermo15_getAmbientTemperatureData( _THERMO15_TEMP_IN_CELSIUS );
    String(Temperature).toCharArray(demoText,10);
    Serial.print("** Temperature : ");
    Serial.println(demoText);
   
    Serial.println( " ---------------------------- ");
    
    delay( 1500 );
}


void setup() 
{
  systemInit();
  applicationInit();
}

void loop() 
{
  applicationTask();
}